<?php

/**
 * Soosyze Framework https://soosyze.com
 *
 * @license https://github.com/soosyze/framework/blob/master/LICENSE (MIT License)
 */

namespace Soosyze\Components\Form;

/**
 * Créer un formulaire conforme aux spécificités de HTML5.
 *
 * @see http://www.w3schools.com/html/html_forms.asp
 *
 * @author Mathieu NOËL <mathieu@soosyze.com>
 */
class FormBuilder extends FormGroupBuilder
{
    /**
     * Déclare l'ouverture du formulaire.
     *
     * @param array $attr
     */
    public function __construct(array $attr = [])
    {
        $this->openForm($attr);
    }

    /**
     * Le formulaire au format HTML.
     *
     * @return string
     */
    public function __toString()
    {
        $html          = $this->form_open() . $this->render() . $this->form_close();
        self::$errors  = [];
        self::$success = [];

        return $html;
    }

    /**
     * Enregistre l'ouverture du formulaire.
     *
     * @param array|null $attr Attributs de la balise form.
     *
     * @return $this
     */
    public function openForm(array $attr = [])
    {
        $this->form[ 'open' ] = [ 'type' => 'open', 'attr' => $attr ];

        return $this;
    }

    /**
     * Génère une balise form fermante.
     *
     * @return string HTML
     */
    public function form_close()
    {
        return '</form>' . self::EOL;
    }

    /**
     * Génère une balise formulaire ouvrante.
     *
     * @param array|null $attrAdd Liste des attributs additionnels.
     *
     * @return string HTML
     */
    public function form_open(array $attrAdd = [])
    {
        $attr = $this->merge_attr($this->form[ 'open' ][ 'attr' ], $attrAdd);

        return '<form' . $this->renderAttrInput($attr) . '>' . self::EOL;
    }
}
